//<c>   Copyright 1992-1999 by Gerry J. Danen; all rights reserved.

//<t>   Information about Logged-In Users

//      Compiled with Microsoft C 8.00. Requires DANEN library to recompile.
//      Source released into the Public Domain.

//  **  Bindery-Emulation only...  **

#include"gd_tools.h"
#include"gd_nw.h"
#include<nit.h>
#include<niterror.h>


#define pgVERS_ "V4.00"
#define pgTITL_ "User Information for Server"
#define pgCOPR_ "1992"


#define MAX_USER 500
typedef struct
{
        UCHAR   name[24] ;
        UCHAR   station[13] ;
        UCHAR   date[22] ;
        ULONG   obj ;
        int     con ;
}       user_t ;
user_t  users[MAX_USER+2] ;


int     no_of_entries ;
INT16   sort_seq = -1 ;
BOOL    quiet_mode=FALSE ;
UCHAR   work[256] ;
char    serverName[48] ;


void    do_help( void )
{
    logon(1);
    printf( "\nUsage:\n\t%s [switches]\n\n"
            "\twhere [switches] can be:\n"
            "\t\t\t /u = sort by user name\n"
            "\t\t\t /s = sort by station\n"
            "\t\t\t /o = sort by object ID\n"
            "\t\t\t /q = quiet: no headers/stats\n"
            "\t\t\t /? or /h = help\n\n"
            "\tOnly one sort switch is allowed\n\n",
            get_global_program_name() );
    exit( BAD_EXIT );
}


void    do_parm( STRING par )
{
    UCHAR       p ;

    _strupr( par );
    while ( *par )
    {
        p = *par++ ;
        switch ( p )
        {
            case 'U' :
                sort_seq = 1 ;
                break ;
            case 'S' :
                sort_seq = 2 ;
                break ;
            case 'O' :
                sort_seq = 3 ;
                break ;
            case 'Q' :
                quiet_mode = TRUE ;
                break ;
            case '?' :
            case 'H' :
                do_help();
            default  :
                fprintf( stderr, "\nSwitch \"%c\" is not legal.\a", p );
                do_help();
        }
    }
}


int     su_compare_nam( user_t *user1, user_t *user2 )
{
    return( strcmp( user1->name, user2->name ) );
}

int     su_compare_stn( user_t *user1, user_t *user2 )
{
    return( strcmp( user1->station, user2->station ) );
}

int     su_compare_obj( user_t *user1, user_t *user2 )
{
    if ( user1->obj < user2->obj )
        return( -1 );
    if ( user1->obj > user2->obj )
        return( 1 );
    return( 0 );
}

void    sort_users( void )
{
    size_t      elements=no_of_entries, element_size=sizeof(users[0]) ;

//  1   /u = sort by user name
//  2   /s = sort by station
//  3   /o = sort by object ID
    switch( sort_seq )
    {
        case 1 :
            qsort( (void *) users, (size_t) elements, (size_t) element_size, su_compare_nam );
            break ;
        case 2 :
            qsort( (void *) users, (size_t) elements, (size_t) element_size, su_compare_stn );
            break ;
        case 3 :
            qsort( (void *) users, (size_t) elements, (size_t) element_size, su_compare_obj );
            break ;
    }
}


void    main( int argc, char *argv[] )
{
    int         ccode, conn, this_station, connect_supported,
                not_logged_in=0, logged_in_users=0 ;
    long        object ;
    WORD16      type ;
    INT32       log_date, log_time ;
    UCHAR       name[49], login_day[4], login_date[12], login_time[8], mark ;
    BYTE        netNum[9], netNode[13], netSock[5] ;
    STRING      hdrfmt = "%-23.23s  %-12.12s  %-21.21s  %-8.8s  %-7.7s\n" ;
    STRING      hdrline= "" ;
    STRING      dots   = ".............................." ;
    STRING      detfmt = "%-23.23s  %-12.12s  %s  %08lX %6d %c\n" ;

    tt_init( SHAREWARE_VERSION, 0, pgVERS_, pgTITL_, pgCOPR_ );
    {
        char s_[133] ;
        strcpy( serverName, nw_server_name( nw_get_default_connection_id() ) );
        sprintf( s_, "%s %s", pgTITL_, serverName );
        SetGlobalString( PGM_TITLE, s_ );
    }
    logon(0);

    if ( ! nw_installed_NetWare() )
    {
        logon(1);
        printf( "NetWare shell is not loaded!\n" );
        exit( BAD_EXIT );
    }
    nw_get_username( name );
    if ( ! strlen(name) )
    {
        logon(1);
        printf( "You must login first!\n" );
        exit( BAD_EXIT );
    }

    this_station = GetConnectionNumber();
    connect_supported = nw_maximum_users();     // find out how many connections supported...

    argc -= 1 ;
    LOOP( conn, 1, argc, 1 )                    // process switches
    {
        ST_CPY( name, argv[conn] );
        if ( name[0] == '/' )
            do_parm( &name[1] );
    }

    if ( ! quiet_mode )
    {
        printf( "Server %s\n", serverName );
        printf( hdrfmt, "User", "Station", "Login Time", "ObjectID", "Connect" );
        printf( hdrfmt, hdrline, hdrline, hdrline, hdrline, hdrline );
    }
    no_of_entries = 0 ;

    LOOP( conn, 1, connect_supported, 1 )
    {
        ccode = nw_get_conn_info( conn, name, &type, &object, &log_date, &log_time, netNum, netNode, netSock );
        if ( ! ccode )
        {
            if ( *netNode == '?' )              // v3.10
                strcpy( netNode, "Unknown     " );
            if ( st_eq(name, "not-logged-in") )
            {
                not_logged_in += 1 ;
            }
            else
            {
                if ( conn == this_station )
                    mark = '*' ;
                else
                    mark = ' ' ;
                FormatDate( log_date, login_date, 1 );
                WhatDay   ( log_date, login_day , 0 );
                FormatTime( log_time, login_time, 0, 0 );
                sprintf( work, "%s %s %s", login_day, login_date, login_time );
                if ( ! quiet_mode )
                {
                    ST_CAT( name, " " );
                    ST_CAT( name, dots);
                }
                if ( sort_seq < 0 )
                {
                    printf( detfmt, name, netNode, work, object, conn, mark );
                }
                else
                {
                    st_cpy( users[no_of_entries].name, name, 24 );
                    st_cpy( users[no_of_entries].station, netNode, 13 );
                    st_cpy( users[no_of_entries].date, work, 22 );
                    users[no_of_entries].obj = object ;
                    users[no_of_entries].con = conn ;
                    no_of_entries += 1 ;
                }
                logged_in_users += 1 ;
            }
        }
    }
    if ( sort_seq > 0 )
    {
        sort_users();
        no_of_entries -= 1 ;
        LOOP( conn, 0, no_of_entries, 1 )
        {
            if ( users[conn].con == this_station )
                mark = '*' ;
            else
                mark = ' ' ;
            printf( detfmt, users[conn].name,
                            users[conn].station,
                            users[conn].date,
                            users[conn].obj,
                            users[conn].con, mark );
        }
    }
    if ( ! quiet_mode )
    {
        conn = connect_supported - logged_in_users ;
        printf( "%d User%s logged in.  %d Free connection%s",
                logged_in_users, (logged_in_users==1 ? "" : "s"),
                conn, (conn==1 ? "" : "s") );
        if ( not_logged_in )
            printf( ", including %d NOT-LOGGED-IN", not_logged_in );
        printf( ".\n" );
    }
    exit( GOOD_EXIT );
}
